import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;

/**
 * A simple program that automatically generates implementations 
 * for the feature model based on the templates and the model.xml file.
 * Parses the model file using jsoup and creates a Main Configurator
 * and a .jak for each module.
 *  
 */


public class ImplGenerator {
	public static void main(String[] aArgs) {
		ImplGenerator oGenerator = new ImplGenerator();
	}
	
	//input path for the model file
	String m_sFeatureModelPath = "../FeatureModelConfigurator/model.xml";
	//input path for the main configurator template
	String m_sMainTemplatePath = "./template/Main.jak";
	//input path for the module template
	String m_sModuleTemplatePath = "./template/Module.jak";
	//input path for the adapter
	String m_sAdapterPath = "./src/Adapter.java";
	
	private ImplGenerator() {
		//create document from model file
		Document oFeatureModelDoc;
		try {
			oFeatureModelDoc = Adapter.readXml(m_sFeatureModelPath);
		}
		catch (Exception oEx) {
			System.out.print(oEx);
			return;
		}
		//create main configurator file
		String sDestPath = "../FeatureModelConfigurator/features/ConfiguratorMain/Configurator.jak";
		CopyFile(m_sMainTemplatePath, sDestPath);
		
		//get modules (children of the Configurator Main)
		List<Element> aModules = Adapter.xQueryElements(oFeatureModelDoc, "//*[@name= 'ConfiguratorMain']").get(0).getChildren();
		FindNonAbstractModules(aModules);
		
		//copy adapter file
		sDestPath = "../FeatureModelConfigurator/features/ConfiguratorMain/Adapter.java";
		CopyFile(m_sAdapterPath, sDestPath);
	}
	
	//recursively iterate through modules, create files for all non-abstract modules
	private void FindNonAbstractModules(List<Element> aModules) {
		for(Element oModule : aModules) {
			//if the Element is not a graphics node
			if(oModule.getName().equals("graphics")) {
				continue;
			}
			//check if the Attributes of the Module contain "abstract"
			List<Attribute> aAttributes = oModule.getAttributes();
			boolean bAbstract = false;
			for(Attribute oAttr : aAttributes) {
				if(oAttr.getName().equals("abstract")&&oAttr.getValue().equals("true")) {
					bAbstract = true;
					break;
				}				
			}
			if(!bAbstract) {
				String sModuleName = oModule.getAttribute("name").getValue();
				if(sModuleName==null) {
					System.out.println("Couldn't get Module name");
					return;
				}
				String sTargetPath = "../FeatureModelConfigurator/features/"
						+ sModuleName + "/Configurator.jak";
				//copy the file
				CopyFile(m_sModuleTemplatePath, sTargetPath);
				//edit the file and add the name
				Path path = Paths.get(sTargetPath);				
				try {
					//write to file
					String sContent = new String(Files.readAllBytes(path), StandardCharsets.UTF_8);
					sContent = sContent.replaceAll("ModuleName", sModuleName);
					Files.write(path, sContent.getBytes(StandardCharsets.UTF_8));
				}
				catch (IOException oEx) {
					System.out.print(oEx.toString());
				}
			}
			//if the module has children, iterate through them
			if(oModule.getChildren()!=null) {
				FindNonAbstractModules(oModule.getChildren());
			}
		}
	}
	
	//copy a file from source to dest
	private void CopyFile(String sSource, String sDest) {
		File oSourceFile = new File(sSource);
		File oDestFile = new File(sDest);
		
		try {
		
			//check if parent directory exists
			if(!Files.exists(oDestFile.getParentFile().toPath())) {
				//if not, create it
				boolean bCreated = oDestFile.getParentFile().mkdirs();
				if(!bCreated) {
					System.out.println("directory " + oDestFile.getParent() + " could not be created.");
					return;
				}
			}
			
			FileInputStream fis = new FileInputStream(oSourceFile);
			FileOutputStream fos = new FileOutputStream(oDestFile);

	        byte[] aBuffer = new byte[1024];
	        int nLength;

	        while ((nLength = fis.read(aBuffer)) > 0) {
	            fos.write(aBuffer, 0, nLength);
	        }
	            
	        fis.close();
	        fos.close();
	        
	        System.out.println("copied " + sSource + " to " + sDest);
	    }
		catch (IOException oEx) {
			System.out.print(oEx);
		}
	}
}
